from django.core.cache import get_cache
from django.conf import settings

from exchange.models import ExchangeRate


CACHE_ENABLED_KEY = 'EXCHANGE_CACHE_ENABLED'
CACHE_ENABLED_DEFAULT = True

CACHE_DATABASE_KEY = 'EXCHANGE_DATABASE'
CACHE_DATABASE_DEFAULT = 'default'

CACHE_KEY_PREFIX_KEY = 'EXCHANGE_CACHE_KEY_PREFIX'
CACHE_KEY_PREFIX_DEFAULT = 'exchange'

CACHE_DATABASE = getattr(settings, CACHE_DATABASE_KEY, CACHE_DATABASE_DEFAULT)
CACHE_ENABLED = getattr(settings, CACHE_ENABLED_KEY, CACHE_ENABLED_DEFAULT)
CACHE_KEY_PREFIX = getattr(settings, CACHE_KEY_PREFIX_KEY,
                           CACHE_KEY_PREFIX_DEFAULT)

CACHE_TIMEOUT = 0  # Not configurable at all

cache = get_cache(CACHE_DATABASE)


def _get_cache_key(source_currency, target_currency):
    return ':'.join([CACHE_KEY_PREFIX, source_currency, target_currency])


def update_rates_cached():
    rates = ExchangeRate.objects.all()
    cache_map = {_get_cache_key(rate.source.code, rate.target.code): rate.rate
                 for rate in rates}
    cache.set_many(cache_map, timeout=CACHE_TIMEOUT)
    return cache_map


def get_rate_cached(source_currency, target_currency):
    return cache.get(_get_cache_key(source_currency, target_currency))


def get_rates_cached(args_list):
    key_map = {_get_cache_key(*args): args for args in args_list}
    cache_map = cache.get_many(key_map.keys())
    return {key_map[key]: cache_map.get(key) for key in key_map.keys()}



import logging

from exchange.models import Currency, ExchangeRate


logger = logging.getLogger(__name__)


class BaseAdapter(object):
    """Base adapter class provides an interface for updating currency and
    exchange rate models

    """
    def update(self):
        """Actual update process goes here using auxialary ``get_currencies``
        and ``get_exchangerates`` methods. This method creates or updates
        corresponding ``Currency`` and ``ExchangeRate`` models

        """
        currencies = self.get_currencies()
        for code, name in currencies:
            _, created = Currency.objects.get_or_create(
                code=code, defaults={'name': name})
            if created:
                logger.info('currency: %s created', code)

        for source in Currency.objects.all():
            exchange_rates = self.get_exchangerates(source.code) or []
            for code, rate in exchange_rates:
                try:
                    target = Currency.objects.get(code=code)
                    exchange_rate = ExchangeRate.objects.get(source=source,
                                                             target=target)
                    exchange_rate.rate = rate
                    exchange_rate.save()
                    logger.info('exchange rate updated %s/%s=%s'
                                % (source, target, rate))
                except ExchangeRate.DoesNotExist:
                    exchange_rate = ExchangeRate.objects.create(
                        source=source, target=target, rate=rate)
                    logger.info('exchange rate created %s/%s=%s'
                                % (source, target, rate))

    def get_currencies(self):
        """Subclasses must implement this to provide all currency data

        :returns: currency tuples ``[(currency_code, currency_name),]``
        :rtype: list

        """
        raise NotImplemented()

    def get_exchangerates(self, base):
        """Subclasses must implement this to provide corresponding exchange
        rates for given base currency

        :returns: exchange rate tuples ``[(currency_code, rate),]``
        :rtype: list

        """
        raise NotImplemented()



from mock import patch

from django.test import TestCase

from exchange.models import Currency, ExchangeRate
from exchange.adapters.openexchangerates import OpenExchangeRatesAdapter


class OpenExchangeRatesAdapterTest(TestCase):

    def test_update(self):
        with patch(
            'exchange.adapters.openexchangerates.OpenExchangeRatesClient')\
                as client:
            currency_dict = {
                'AED': 'United Arab Emirates Dirham',
                'AFN': 'Afghan Afghani',
                'USD': 'USA'
            }
            latest_dict = {
                'disclaimer': "<Disclaimer data>",
                'license': "<License data>",
                'timestamp': 1358150409,
                'base': "USD",
                'rates': {
                    'AED': 3.66,
                    'AFN': 51.22,
                    'USD': 104.74
                }
            }
            client.return_value.currencies.return_value = currency_dict
            client.return_value.latest.return_value = latest_dict
            adapter = OpenExchangeRatesAdapter()
            adapter.update()
            for k, v in currency_dict.items():
                try:
                    Currency.objects.get(code=k)
                except Currency.DoesNotExist, detail:
                    self.fail(detail)
            for k, v in latest_dict['rates'].items():
                try:
                    ExchangeRate.objects.get(
                        source__code='USD',
                        target__code=k,
                        rate=str(v))
                except ExchangeRate.DoesNotExist, detail:
                    self.fail(detail)



from setuptools import setup, find_packages

setup(
    name='django-exchange',
    packages=find_packages(),
    version='0.2',
    description='currency, exchange rates and conversions support for django',
    author='Metglobal',
    author_email='kadir.pekel@metglobal.com',
    url='https://github.com/metglobal/django-exchange',
    extras_require={
        'openexchangerates': ["openexchangerates"]
    }
)
